#!/usr/bin/python

#  (Standard-)GUI-Modul
#  Datei :  tkmodule.py
#  Autor :  Bombei, BICS, Feb 2003 (Python/Tkinter[pack])
#  Aufgabe   :  stellt Bedienelemente fuer GUI-Steuerungen
#               mit Python/Tkinter bereit


import  Tkinter
from    Tkconstants     import  *

class XHilfsRahmen(Tkinter.Frame):
   """
   /* GUI-Darstellung eines Hilfsrahmens fuer grobe Einteilung
   """
   def __init__(self, Win, Seite=TOP):
      """
      /* Effekt: Ein Hilfsrahmen wird angelegt,
      /* Win kennzeichnet das Parent-Objekt, in dem er sitzen soll.
      /* Seite: optionaler Parameter,
      /*        um Rahmen auch nebeneinander anordnen zu knnen.
      """
      Ausbreitung = BOTH
      Tkinter.Frame.__init__(self, Win)
      self.pack(side=Seite, fill=Ausbreitung, expand=1)

class XThemenRahmen(Tkinter.Frame):
   """
   /* GUI-Darstellung eines beschrifteten Rahmens
   """
   def __init__(self, Win, derText, Seite=TOP):
      """
      /* Effekt: Ein beschrifteter Rahmen wird angelegt,
      /* Win kennzeichnet das Parent-Objekt, in dem er sitzen soll,
      /* derText kennzeichnet die Beschriftung (oberhalb).
      /* Seite: optionaler Parameter,
      /*        um Rahmen auch nebeneinander anordnen zu knnen.
      """
#      if Seite == TOP:
#        Ausbreitung = X
#      else:
#        Ausbreitung = Y
      Ausbreitung = BOTH
      HilfRahmen   = Tkinter.Frame(Win, relief=SUNKEN, bd=2)
      HilfRahmen.pack(side=Seite, padx=4, pady=4, fill=Ausbreitung, expand=1)
      LabelThRahmen = Tkinter.Label(HilfRahmen, text=derText)
      LabelThRahmen.pack()
      Tkinter.Frame.__init__(self, HilfRahmen, relief=RIDGE, bd=2)
      self.pack(padx=2, pady=2, fill=Ausbreitung, expand=1)

class XEingabe:
   """
   /* GUI-Darstellung einer Dateneingabe
   """
   def __init__(self, Win, derText, Bef="pass", Seite=TOP):
      """
      /* Effekt: Ein (Text-) Eingabefeld wird angelegt,
      /* Win kennzeichnet das Parent-Objekt, in dem es sitzen soll.
      /* derText kennzeichnet die Beschriftung (links).
      /* Bef: option. Parameter, noch nicht implementiert ...
      /* Seite: option. Parameter, zum Rahmen nebeneinander Anordnen
      """
      tWert=Tkinter.StringVar()
      HilfRahmen   = Tkinter.Frame(Win)
      LblEingabe = Tkinter.Label(HilfRahmen, text=derText)
      self.Eingabe=Tkinter.Entry(HilfRahmen,textvariable=tWert, width=15)
      HilfRahmen.pack(side=Seite, fill=X, expand=1)
      self.Eingabe.pack(side=RIGHT, padx=2, pady=2)
      LblEingabe.pack(side=RIGHT)

   def get_xEingabe(self):
      """
      /* liefert den eingegebenen Wert aus dem Text-Eingabefeld,
      /* bei fehlender Eingabe: ein Leerzeichen!
      """
      Eing = self.Eingabe.get()
      if Eing == "" :
           Eing = " "
      return Eing

   def set_xEingabe(self,wert):
      """
      /* Belegung des Text-Eingabefeldes mit einem Wert
      """
      self.Eingabe.delete(0,END)
      self.Eingabe.insert(0,wert)

class XTaste:
   """
   /* GUI-Darstellung einer Taste
   """
   def __init__(self, Win, derText, Bef, Seite=LEFT):
      """
      /* Effekt: Eine beschriftete Taste wird angelegt,
      /* Win kennzeichnet das Parent-Objekt, in das sie gesetzt wird.
      /* derText : string : Beschriftung der Taste
      /* Bef  : Methodenreferenz : Methode, die beim Drcken der Taste
      /*                           ausgefhrt werden soll.
      /* Seite: optionaler Parameter,
      /*        um Tasten auch bereinander anordnen zu knnen
      """
      Knopf=Tkinter.Button(Win, text=derText, command=Bef)
      Knopf.pack(side=Seite, fill=X, expand=1)

class XDialog:
    """
    /* GUI-Darstellung einer Dialogbox
    """
    def __init__(self,derText):
      """
      /* Effekt  : Eine beschriftete Dialogbox ("Weiter") wird angelegt,
      /*      aber nicht gezeigt!
      /* derText : string : Beschriftung der Box
      """
      self.extraRahmen   = Tkinter.Toplevel()
      self.extraRahmen.withdraw()
      self.extraRahmen.geometry ("+50+120")
      extraLabel  = Tkinter.Label(self.extraRahmen,text=derText,bg="red")
      extraKnopf  = Tkinter.Button(self.extraRahmen,text="Weiter",command=self.extraRahmen.destroy)
      extraLabel.pack()
      extraKnopf.pack()
    def zeige(self):
      """
      /* Effekt  : Die zuvor angelegte Dialogbox ("Weiter") wird anzeigt!
      /*           Bei Bettigen der "Weiter"-Taste verschwindet die Box.
      """
      self.extraRahmen.deiconify()

class XWahl:
    """
    /* GUI-Darstellung einer Radio-Button-Leiste
    """
    def __init__(self,Win,derText,liste,Seite=TOP):
        """
        /* Erzeugt Knopfleiste entsprechend der bergebenen Werte-"liste"
        /* Win kennzeichnet das Parent-Objekt, in das sie gesetzt wird.
        """
        dieSeite      = Seite
        derRahmen     = XThemenRahmen(Win,derText,dieSeite)
	self.gewaehlt = Tkinter.StringVar()
	self.alleWerte = liste
	for dieserWert in liste:
		dieWahl = Tkinter.Radiobutton(derRahmen,
			text     = dieserWert,
			variable = self.gewaehlt,
			value    = dieserWert)
		dieWahl.pack(side = Seite,ipadx=4,ipady=4)

    def get_xEingabe(self):
      """
      /* liefert den Wert der gedrckten Taste,
      /* bei fehlender Eingabe: ein Leerzeichen!
      """
      Eing = self.gewaehlt.get()
      if Eing == "" :
           Eing = " "
      return Eing

    def set_xEingabe(self,wert):
      """
      /* Knopf markieren, entsprechend bergebenem "wert"
      """
      if wert == " ":
        self.gewaehlt.set(NONE)
      else:
        self.gewaehlt.set(wert)

class XListe :
    """
    /* GUI-Darstellung einer scrollbaren Mehrfach-Auswahl-Liste
    """
    def __init__ (self, derText, liste=[], Bef="None"):
        """
        /* Erzeugt Listbox entsprechend der bergebenen Werte-"liste",
#        /* ... muss mit "zeige" sichtbar gemacht werden!
        /* derText : string : Beschriftung der Box
        """
        self.Inhalt = liste
        self.extraRahmen   = Tkinter.Toplevel()
        self.extraRahmen.title(" List-Box ")
        self.extraRahmen.transient()
        self.extraRahmen.withdraw()
        if Bef == "None":
          Befehl = self.extraRahmen.iconify
        else :
          Befehl = Bef
        listRahmen  = XThemenRahmen(self.extraRahmen,derText)
        listRahmen.pack()
        rollBalken = Tkinter.Scrollbar(listRahmen, orient=VERTICAL)
        self.listBox = Tkinter.Listbox(listRahmen,yscrollcommand=rollBalken.set,selectmode=EXTENDED)
        rollBalken.config(command=self.listBox.yview)
        rollBalken.pack(side=RIGHT, fill=Y)
        self.listBox.pack(side=LEFT, fill=BOTH, expand=1)
        listOkKnopf  = Tkinter.Button(self.extraRahmen,text="OK",command=Befehl)
        listOkKnopf.pack(side=LEFT)
        listEndeKnopf  = Tkinter.Button(self.extraRahmen,text="Schlieen",command=self.extraRahmen.iconify)
        listEndeKnopf.pack(side=RIGHT)

        for element in self.Inhalt:
          self.listBox.insert(END, element)

    def get_xAuswahl(self):
      """
      /* liefert eine Liste der gewaehlten Elemente,
      /* bei leerer Box oder fehlender Wahl: eine leere Liste
      """
      Auswahl = []
      welche  = self.listBox.curselection()
      for index in welche:
        Auswahl.append(self.listBox.get(int(index)))
      return Auswahl

    def zeige(self):
      """
      /* Effekt  : Die zuvor angelegte Listenbox wird angezeigt!
      /* Bei Bettigen der "Beende"-Taste verschwindet die Box.
      """
      self.extraRahmen.deiconify()
      self.listBox.see(0)

    def fuelle(self,liste):
      """
      /* Effekt  : In der Listenbox wird die "liste" angezeigt!
      """
      self.listBox.delete(0, END)
      self.Inhalt = liste
      for element in self.Inhalt:
            self.listBox.insert(END, element)
#      self.listBox.see(0)

