#!/usr/bin/python

# Projekt               : Listenverwaltung
# K L A S S E           : TListe
# Datei                 : sliste.py
# Angepasst fuer Python : August 02, Bombei, BICS
# --------------------------------------------------------------------- #
# Aufgabe       : Allgemeine statische Liste zur Verwaltung beliebiger  #
#                 Objekte.                                              #
#                 Zum Bewegen in der Liste ist intern eine Listenmarke  #
#                 mitgefuehrt, die die jeweils aktuelle Listenposition  #
#                 bezeichnet. Alle Operationen werden auf der aktuellen #
#                 Position ausgefuehrt.                                 #
#                                                                       #
# Version       : 0.1.04 (Turbo-Pascal 7.0 - Delphi 4.0)                #
# Autor         : S. Spolwig, OSZ-Handel I, 10997 Berlin                #
# ********************************************************************* #

import pickle

class TListe:
   """
   /* Allgemeine statische Liste zur Verwaltung beliebiger Objekte.
   /* Zum Bewegen in der Liste ist intern eine Listenmarke
   /* mitgefuehrt, die die jeweils aktuelle Listenposition bezeichnet.
   /* Alle Operationen werden auf der aktuellen  Position ausgefuehrt.                                 #
   """

   def __init__(selbst):
        """
        /* Auftrag : Listenobjekt erzeugen und initialisieren
        /* vorher  : -
        /* nachher : Lilaenge ist Null, AktuellePos ist 0.
        """
        selbst.Kollektion  = []
        selbst.Lilaenge    = 0
        selbst.AktuellePos = 0

   def First(selbst):
        """
        /* Auftrag : Listenmarke auf das 1. Element setzen
        /* vorher  : Die Liste ist initialisiert und nicht leer.
        /* nachher : Aktuelle Position ist 1
        /*           Wenn IsEmpty , geschieht nichts.
        """
        if ( not selbst.IsEmpty() ) :
          selbst.AktuellePos = 1

   def Next(selbst):
        """
        /* Auftrag : Die Listenmarke auf die naechste Position setzen.
        /* vorher  : Die Liste ist initialisert
        /* nachher : Die Marke steht auf der neuen aktuellen Position
        /*           Wenn EoList erreicht ist, geschieht nichts.
        """
        if ( not selbst.EoList() ) :
          selbst.AktuellePos = selbst.AktuellePos + 1

   def Previous(selbst):
        """
        /* Auftrag : Die Listenmarke um eins zuruecksetzen
        /* vorher  : Die Liste ist nicht leer (wird nicht geprueft)
        /* nachher : Die Marke steht auf der neuen aktuellen Position.
        /*           Wenn die aktuelle Position 1 war, geschieht nichts
        """
        if  selbst.AktuellePos > 1 :
            selbst.AktuellePos = selbst.AktuellePos - 1

   def Last(selbst):
        """
        /* Auftrag : Listenmarke auf das letzte Element der Liste setzen
        /* vorher  : Die Liste ist nicht leer (wird nicht geprueft).
        /* nachher : Letztes vorhandenes Element ist aktuelle Position
        """
        selbst.AktuellePos = selbst.Lilaenge

   def GetElement (selbst):
        """
        /* Anfrage : Zugriff auf das aktuelle Element zum Lesen
        /* vorher  : Die Liste ist nicht leer.
        /* nachher : GetElement liefert das aktuelle Element.
        /* Fehler  : liefert "0", wenn AktuellePos vor oder hinter Liste
        """
        if ( not selbst.EoList() and selbst.AktuellePos > 0 ) :
          return selbst.Kollektion[selbst.AktuellePos-1]
        else :
          return 0

   def PutElement (selbst, Elem):
        """
        /* Auftrag : Zugriff auf die aktuelle Position zum Schreiben
        /* vorher  : Die Liste ist initialisiert.
        /* nachher : Elem ist an aktuelle Position uebergeben
        /*           , ausser wenn AktuellePos vor oder hinter Liste
        """
        if ( not selbst.EoList() and selbst.AktuellePos > 0 ) :
          selbst.Kollektion[selbst.AktuellePos-1] = Elem;

   def Append (selbst, Elem):
        """
        /* Auftrag : Neues Element an das Ende der Liste anhaengen.
        /* Wenn die Liste leer war, ist das neue El. das erste.
        /* vorher  : Die Liste ist initialisiert
        /* nachher : Listenlaenge ist um eins erhoeht. Position am Ende.
        """
        selbst.Last();
        selbst.AktuellePos  =  selbst.AktuellePos + 1
        selbst.Kollektion.append(Elem)
        selbst.Lilaenge    =  selbst.Lilaenge + 1

   def EoList(selbst):
        """
        /*
        /* Anfrage: Pruefen, ob das Ende der Liste erreicht ist.
        /* vorher : Die Liste ist initialisiert.
        /* nachher: "1", wenn Ende der Liste erreicht ist, d.h. die
        /*          Listenmarke hinter dem letzten belegten Element steht,
        /*          sonst "0"
        """
        if selbst.AktuellePos > selbst.Lilaenge :
            return 1
        else :
            return 0

   def IsEmpty(selbst):
        """
        /* Anfrage: ob Liste leer ist
        /* vorher : Die Liste ist initialisiert.
        /* nachher: "1", wenn die Liste leer ist, sonst "0".
        """
        if selbst.Lilaenge == 0 :
            return 1
        else :
            return 0

   def PositionOf (selbst, suchElem) :
        """
        /* Effekt : liefert die (1.) Position des "suchElem" in der Liste,
        /* Fehler : liefert "0", wenn nicht vorhanden oder Liste leer.
        /* benutzt: Listenoperation "index" liefert erste Position
        /*          eines "gleichen" Elements.
        """
        try :
          Result = selbst.Kollektion.index(suchElem)+1
#        except ValueError :  # bzw. zum Abfangen _aller_ Fehler:
        except :
          Result = 0
        return Result

   """
#   def SearchFor (suchElem) :
#        /* ( benutzt: Elem.IstGleich(suchElem):
#        /*          Vergleichsoperation der Elementklasse!!! )
#        /* ( Was soll diese Methode eigentlich bewirken? )
        altePos  = selbst.AktuellePos
        Result   = 0
        gefunden = 0
        selbst.First()

        while NOT (gefunden OR selbst.EoList()) :
          Elem = selbst.GetElement()
          if Elem.IstGleich(suchElem) :
                gefunden = 1
                Result = selbst.AktuellePos
                #                Result = Elem
          else :
            selbst.Next()

        selbst.AktuellePos = altePos
        return Result
   """

   def Load (selbst, Dateiname):
        """
        /* Auftrag: Liste aus externer Datei laden.
        /* vorher : Die Liste ist initialisiert, Datei existiert .
        /* nachher: Die Liste ist in den Arbeitsspeicher geladen;
        /*          Datei ist geschlossen; AktPos ist auf erstem Element;
        /*          Lilaenge entspricht der Listenlaenge, Rueckgabewert "1"
        /* Fehler : misslingt das Oeffnen der Datei, geschieht nichts,
        /*          Rueckgabewert "0"
        /* bentigt das Modul "pickle" (siehe "import")
        """
        try :
          Fs = open(Dateiname, "r")
        except :
          return 0
        else :
#            einPack = pickle.Pickler(Fs)
#            einPack.dump(selbst.Kollektion)
          selbst.Kollektion  = pickle.Unpickler(Fs).load()
          Fs.close()
          selbst.Lilaenge   = len (selbst.Kollektion)
          selbst.First()
          return 1

   def Store(selbst, Dateiname):
        """
        /* Auftrag: Liste in externe Datei speichern
        /* vorher : -
        /* nachher: Die Liste ist unter "Dateiname" gespeichert.
        /*          Die Datei ist geschlossen. Rueckgabewert "1".
        /* Achtung: Evtl. existierende Dateien werden ueberschrieben!!
        /* Fehler : Misslingt das Oeffnen der Datei, geschieht nichts.
        /*          Ist die Liste leer, wird nichts gespeichert.
        /*          Rueckgabewert "0"
        /* bentigt das Modul "pickle" (siehe "import")
        """
        if not selbst.IsEmpty() :
          try :
            Fs = open(Dateiname, "w")
          except :
            return 0
          else :
            einPack = pickle.Pickler(Fs)
            einPack.dump(selbst.Kollektion)
#            selbst.First()
#            while not selbst.EoList() :
#              Fs.write(selbst.GetElement())
#              Fs.write("\n") # das bekommt man nachher nicht weg !!!
#              selbst.Next()
#            Fs.writelines(selbst.Kollektion) # trennt die Datenstze nicht
            Fs.close()
            return 1
        else :
            return 0


#
#  Testprogramm
#
if __name__ == '__main__':

  dL    = TListe()
  zahl  = 0
  maxzahl = 14
  zListe  = range (zahl,maxzahl)
  kListe  = range (zahl,int(maxzahl/2))
  print "\nListentest\n\nAusgangswerte: ",
  print "von: %d  bis: %d " % (zahl, maxzahl)
  print "... daraus eine Zahlenfolge: ",
  print zListe
#  print dL
  if dL.IsEmpty() : print "Liste 'dL' ist leer."
  print "aktuelle Position: %d " % dL.AktuellePos
  print "am Listenende anhngen; Fortschritt:"

  for element in zListe :
    dL.Append(str(element))
    print dL.AktuellePos,dL.Lilaenge,
  print

  eins = dL.GetElement()
  print "das aktuelle (letzte) Element holen u. anzeigen: ",
  print eins

  if dL.IsEmpty() : print "Liste 'dL' ist leer.",
  if dL.EoList()  : print "\nListe 'dL' ist am Ende.",
  print "aktuelle Position: %d " % dL.AktuellePos

  print "die ersten Inhalte gezielt verndern; zeigt Pos-/Element-Paare:"
  dL.First()

  for element in kListe :
    drei = dL.GetElement()
    dL.PutElement(drei+drei)
    print dL.AktuellePos,dL.GetElement(),
    dL.Next()
    if dL.IsEmpty() : print "Liste 'dL' ist leer.",
    if dL.EoList()  : print "\nListe 'dL' ist am Ende.",
  print

  dL.First()

  for element in zListe :
    zwei = dL.GetElement()
    print dL.AktuellePos,zwei,
    dL.Next()
    if dL.IsEmpty() : print "Liste 'dL' ist leer.",
    if dL.EoList()  : print "\nListe 'dL' ist am Ende.",
  print

  if dL.PositionOf(eins) :
    print "Element '",
    print eins,
    print "' gefunden an Position: %d" % dL.PositionOf(eins)

  if dL.Store("SLTest.txt") :
    print "Datei geschrieben: 'SLTest.txt'"
    zL = TListe()
    print "neue Liste (zL):",
    if zL.IsEmpty() : print "Liste 'zL' ist leer.",
    if zL.EoList()  : print "Liste 'zL' ist am Ende.",
    print
    if zL.Load("SLTest.txt") :
      print "Datei gelesen: 'SLTest.txt'; so sieht die Liste (zL) aus:"
      print zL.Kollektion

      zL.First()
      print "... zeigt Pos-/Element-Paare:"

      for element in zListe :
        zwei = zL.GetElement()
        print zL.AktuellePos,zwei,
        zL.Next()
        if zL.IsEmpty() : print "Liste 'zL' ist leer.",
        if zL.EoList()  : print "\nListe 'zL' ist am Ende.",
      print
    else : print "Datei nicht gelesen: 'SLTest.txt':"
  else : print "Datei nicht geschrieben: 'SLTest.txt':"

